---
layout: default
title: Attributes
parent: Deriving JsonSchema
nav_order: 1
permalink: /deriving/attributes/
---

<style>
h3 code {
    font-weight: bold;
}
</style>

# Attributes

You can add attributes to your types to customize Schemars's derived `JsonSchema` implementation.

Serde also allows setting `#[serde(...)]` attributes which change how types are serialized, and Schemars will generally respect these attributes to ensure that generated schemas will match how the type is serialized by serde_json. `#[serde(...)]` attributes can be overriden using `#[schemars(...)]` attributes, which behave identically (e.g. `#[schemars(rename_all = "camelCase")]`). You may find this useful if you want to change the generated schema without affecting Serde's behaviour, or if you're just not using Serde.

<details open>
<summary style="font-weight: bold">
TABLE OF CONTENTS
</summary>

1. [Supported Serde Attributes](#supported-serde-attributes)
    - [`rename`](#rename)
    - [`rename_all`](#rename_all)
    - [`tag` / `content` / `untagged`](#tag)
    - [`default`](#default)
    - [`skip`](#skip)
    - [`skip_serializing`](#skip_serializing)
    - [`skip_deserializing`](#skip_deserializing)
    - [`flatten`](#flatten)
    - [`with`](#with)
1. [Other Attributes](#other-attributes)
    - [`schema_with`](#schema_with)
    - [`title` / `description`](#title-description)
    - [`example`](#example)
    - [`deprecated`](#deprecated)
    - [Doc Comments (`doc`)](#doc)
</details>

## Supported Serde Attributes

<div class="indented">

<h3 id="rename">

`#[serde(rename = "name")]` / `#[schemars(rename = "name")]`
</h3>

Set on a struct, enum, field or variant to use the given name in the generated schema instead of the Rust name. When used on a struct or enum, the given name will be used as the title for root schemas, and the key within the root's `definitions` property for subschemas.

If set on a struct or enum with generic type parameters, then the given name may contain them enclosed in curly braces (e.g. `{T}`) and they will be replaced with the concrete type names when the schema is generated.

Serde docs: [container](https://serde.rs/container-attrs.html#rename) / [variant](https://serde.rs/variant-attrs.html#rename) / [field](https://serde.rs/field-attrs.html#rename)

<h3 id="rename_all">

`#[serde(rename_all = "...")]` / `#[schemars(rename_all = "...")]`
</h3>

Set on a struct, enum or variant to rename all fields according to the given case convention (see the Serde docs for details).

Serde docs: [container](https://serde.rs/container-attrs.html#rename_all) / [variant](https://serde.rs/variant-attrs.html#rename_all)

<h3 id="tag" style="line-height: 1.5">

`#[serde(tag = "type")]` / `#[schemars(tag = "type")]` <br />
`#[serde(tag = "t", content = "c")]` / `#[schemars(tag = "t", content = "c")]` <br />
`#[serde(untagged)]` / `#[schemars(untagged)]`
</h3>

Set on an enum to generate the schema for the [internally tagged](https://serde.rs/enum-representations.html#internally-tagged), [adjacently tagged](https://serde.rs/enum-representations.html#adjacently-tagged), or [untagged](https://serde.rs/enum-representations.html#untagged) representation of this enum.

Serde docs: [`tag`](https://serde.rs/container-attrs.html#tag) / [`tag`+`content`](https://serde.rs/container-attrs.html#tag--content) / [`untagged`](https://serde.rs/container-attrs.html#untagged)

<h3 id="default">

`#[serde(default)]` / `#[schemars(default)]` / `#[serde(default = "path")]` / `#[schemars(default = "path")]`
</h3>

Set on a struct or field to give fields a default value, which excludes them from the schema's `required` properties. The default will also be set on the field's schema's `default` property, unless it is skipped by a [`skip_serializing_if`](https://serde.rs/field-attrs.html#skip_serializing_if) attribute on the field. Any [`serialize_with`](https://serde.rs/field-attrs.html#serialize_with) or [`with`](https://serde.rs/field-attrs.html#with) attribute set on the field will be used to serialize the default value.

Serde docs: [container](https://serde.rs/container-attrs.html#default) / [field](https://serde.rs/field-attrs.html#default)

<h3 id="skip">

`#[serde(skip)]` / `#[schemars(skip)]`
</h3>

Set on a variant or field to prevent it from appearing in any generated schema.

Serde docs: [variant](https://serde.rs/variant-attrs.html#skip) / [field](https://serde.rs/field-attrs.html#skip)

<h3 id="skip_serializing">

`#[serde(skip_serializing)]` / `#[schemars(skip_serializing)]`
</h3>

Set on a field of a (non-tuple) struct to set the `writeOnly` property on that field's schema. Serde also allows this attribute on variants or tuple struct fields, but this will have no effect on generated schemas.

Serde docs: [field](https://serde.rs/field-attrs.html#skip_deserializing)

<h3 id="skip_deserializing">

`#[serde(skip_deserializing)]` / `#[schemars(skip_deserializing)]`
</h3>

Set on a variant or field. When set on a field of a (non-tuple) struct, that field's schema will have the `readOnly` property set. When set on a variant or tuple struct field Schemars will treat this the same as a [`skip`](#skip) attribute.

Serde docs: [variant](https://serde.rs/variant-attrs.html#skip_deserializing) / [field](https://serde.rs/field-attrs.html#skip_deserializing)

<h3 id="flatten">

`#[serde(flatten)]` / `#[schemars(flatten)]`
</h3>

Set on a field to include that field's contents as though they belonged to the field's container.

Serde docs: [field](https://serde.rs/field-attrs.html#flatten)

<h3 id="with">

`#[serde(with = "Type")]` / `#[schemars(with = "Type")]`
</h3>

Set on a variant or field to generate its schema as the given type instead of its actual type. Serde allows the `with` attribute to refer to any module path, but Schemars requires this to be an actual type which implements `JsonSchema`.

If the given type has any required generic type parameters, then they must all be explicitly specified in this attribute. Serde frequently allows you to omit them as it can make use of type inference, but unfortunately this is not possible with Schemars. For example, `with = "Vec::<i32>"` will work, but `with = "Vec"` and `with = "Vec::<_>"` will not.

Serde docs: [variant](https://serde.rs/variant-attrs.html#with) / [field](https://serde.rs/field-attrs.html#with)

<h3 id="deny_unknown_fields">

`#[serde(deny_unknown_fields)]` / `#[schemars(deny_unknown_fields)]`
</h3>

Setting this on a container will set the `additionalProperties` keyword on generated schemas to `false` to show that any extra properties are explicitly disallowed.

Serde docs: [container](https://serde.rs/container-attrs.html#deny_unknown_fields)

<h3 id="transparent">

`#[serde(transparent)]` / `#[schemars(transparent)]`
</h3>

Set on a newtype struct or a braced struct with one field to make the struct's generated schema exactly the same as that of the single field's.

Serde docs: [container](https://serde.rs/container-attrs.html#transparent)

</div>

## Other Attributes

<h3 id="schema_with">

`#[schemars(schema_with = "some::function")]`
</h3>

Set on a variant or field to generate this field's schema using the given function. This function must be callable as `fn(&mut schemars::gen::SchemaGenerator) -> schemars::schema::Schema`.

<h3 id="title-description">

`#[schemars(title = "Some title", description = "Some description")]`
</h3>

Set on a container, variant or field to set the generated schema's `title` and/or `description`. If present, these will be used instead of values from any [`doc` comments/attributes](#doc).

<h3 id="example">

`#[schemars(example = "some::function")]`
</h3>

Set on a container, variant or field to include the result of the given function in the generated schema's `examples`. The function should take no parameters and can return any type that implements serde's `Serialize` trait - it does not need to return the same type as the attached struct/field. This attribute can be repeated to specify multiple examples.

<h3 id="deprecated">

`#[deprecated]`
</h3>

Set the Rust built-in [`deprecated`](https://doc.rust-lang.org/edition-guide/rust-2018/the-compiler/an-attribute-for-deprecation.html) attribute on a struct, enum, field or variant to set the generated schema's `deprecated` keyword to `true`.

<h3 id="doc">

Doc Comments (`#[doc = "..."]`)
</h3>

If a struct, variant or field has any [doc comments](https://doc.rust-lang.org/stable/rust-by-example/meta/doc.html#doc-comments) (or [`doc` attributes](https://doc.rust-lang.org/rustdoc/the-doc-attribute.html)), then these will be used as the generated schema's `description`. If the first line is an ATX-style markdown heading (i.e. it begins with a # character), then it will be used as the schema's `title`, and the remaining lines will be the `description`.
